/**********************************************************************************************************/
/* Replication program for ""Crowding in" Effect of Public Investment on Private Investment Revisited"    */
/* Authored by: John Nana Francois (World Bank), Maty Konte (IFC), and Franz U Ruch (World Bank)     */
/* Contact: jfrancois1@worldbank.org; mkonte@ifc.org and fruch@worldbank.org */ 																										
/***********************************************************************************************************/



gl root "C:\WBG\github\187-v3\FKR2024CrowdingInWB_Sep24 (1)\Reproducibility package"
gl data "${root}\Data"
gl outputs "${root}\Outputs"
sysdir set PLUS "${root}\ado"
set varabbrev on
/*
local packages grstyle palettes colrspace xtscc xtcd2 ivreg2 kinkyreg ranktest asdoc

foreach pkg in `packages' {
    ssc install `pkg'
}
//rivtest
package name:  st0171_1.pkg
from:  http://www.stata-journal.com/software/sj11-2/
*/

do "${root}\ConstructingFinalData.do"

#delimit ;
grstyle init;
clear;clear matrix;
set more off;
set matsize 800;



/*-----------------------*/   
/** Graphing Scheme *****/   
/*-----------------------*/
grstyle init;
grstyle set plain, grid;
grstyle set plain;
grstyle set color Set1;
grstyle set symbol Oh;
grstyle set symbolsize  2 , pt;
grstyle set linewidth .8pt: plineplot;

use "${data}\FKR2024_CrowdingIn.dta", clear;

/****************************************************************************/
/* Key Variable definitions */
/****************************************************************************/
/* 
/* Country and year identifiers */
ctry:     three-letter ISO country code
country:  country name
ifscode:     numerical country code
year:     year
ida: dummy equal to 1 for countries eligible for concessional lending from International Development Association (IDA)

/* Instrumental Variables */
/* Loan-level data in the Debtor Reporting System are confidential. Country-year level aggregates are reported */
/* We adopt the nomenclature of Kraay, A. 2014. "Government Spending Multipliers in Developing Countries: Evidence from Lending by Official Creditors." 
American Economic Journal: Macroeconomics, 6 (4): 170–208. */

disbtot:  total disbursements (including loans approved in the same year)
disbex0-2:  total disbursements excluding loans approved in same year (ex0), same and previous year (ex1) and same and two previous years (ex2)
preddisbtot:  total predicted disbursements (including on loans approved in the same year)
preddisbex0-2: total predicted disbursements excluding loans approved in same year (ex0), same and previous year (ex1) and same and two previous years (ex2)
mpreddisbex0: predicted disbursements instrument for multilateral creditors only
bpreddisbex0: predicted disbursements instrument for bilateral creditors only
preddisb1ex0: predicted disbursements instrument constructed by combining all loans into single bin when calculating typical disbursement rates
preddisb2ex0: predicted disbursements instrument constructed excluding all loans from country in question when calculating typical disbursement rates
slow3sum:  Value of newly-approved projects in given country-year that ultimately take at least four years to disburse fully, current USD  

/* Basic macro data from World Economic Outlook */
weogdpcd: GDP in current USD
weogdpcl: GDP in current LCU
weogdpkl: GDP in constant LCU

/* Dependent and Independent Variable */
igov_n:  General government investment (gross fixed capital formation), in billions of nominal national currency
ipriv_n: Private investment (gross fixed capital formation), in billions of nominal national currency

/* Other  V and Independent Variable */
FD:  Financial Development Index from IMF 
kaopen  : Chinn-Ito Index 
ka_open : Chinn-Ito Index, normalized
cce : Control of Corruption Index (-2.5 to 2.5) [Worldwide Governance Indicators]
*/

/************************************************************************************************************/
/* Identify set of countries where disbursements are large as share of GDP                                  */
/* Set threshold at 1% of GDP. Also restrict sample to countries with at least 15 time series observations  */
/************************************************************************************************************/
gen disbtoty= disbtot/weogdpcd;
summarize disbtoty, d;
egen disbtotyav=mean(disbtoty) if year>=1980 & year<=2019, by(ctry);
egen igovcnt=count(igov_n) if year>=1980 & year<=2019, by(ctry);
summarize disbtotyav if year==2000, d;


gen smpl=0; 
replace smpl=1 if disbtotyav~=. & igovcnt>= 15;

/* Drop all countries below threshold for large disbursements, and redundant years*/
drop if smpl==0 | year<1980 | year>2020;


/* Adjust IDA dummy to drop 8 countries included in IDA under the small island economies exception, as these countries are not LICs */
replace ida=. if ctry=="KIR" | ctry=="CPV" | ctry=="TON" | ctry=="VUT" | ctry=="DMA" | ctry=="GRD" | ctry=="LCA" | ctry=="VCT";


/***********************************************************************************************************/
/* Create ratios to nominal GDP and real changes/lagged real GDP of all disbursement variables measured in current USD */
/***********************************************************************************************************/
/* Exchange rate (LCU/$) i.e. use exchange rate used to convert national accounts in WDI */ 
gen xr=weogdpcl/weogdpcd;
/* Local GDP deflator */
gen pgdp=weogdpcl/weogdpkl;

*sort ctry year;

for var disbtot preddisbtot preddisbex0 mpreddisbex0 bpreddisbex0 preddisbex0_rdsum2 preddisbex0_nloans207 preddisbex0_occdy4: 
	gen Xgdp=X/weogdpcd \
	gen dXgdp=(X*xr/pgdp-L.X*L.xr/L.pgdp)/L.weogdpkl;
	
/***********************************************************************************************************/
/* Create ratios to nominal GDP and real changes/real GDP for government and private investment */
/***********************************************************************************************************/
gen igov= igov_n*1000000000; 
gen digovgdp=(igov/pgdp-L.igov/L.pgdp)/L.weogdpkl;


gen ipriv= ipriv_n*1000000000;
gen diprivgdp=(ipriv/pgdp-L.ipriv/L.pgdp)/L.weogdpkl;
gen L1diprivgdp = (L1.ipriv/L1.pgdp-L2.ipriv/L2.pgdp)/L.weogdpkl;
	
/* Real local currency constant price growth rates */
gen dyy=(weogdpkl-L.weogdpkl)/L.weogdpkl; 
/*gen dyy=(GDP_n/pgdp-L.GDP_n/L.pgdp)/(L.GDP_n/L.pgdp); (L.GDP_n*1000000000/L.pgdp)*/


/***********************************************************************************************************/
/* Clean data by dropping extreme observations on changes in govt spending, predicted disbursements, and real GDP (trim at 1 and 99 percentile)  */
/***********************************************************************************************************/
sum dpreddisbex0gdp dyy diprivgdp digovgdp, d;
for var dpreddisbex0gdp dyy diprivgdp digovgdp:
	egen Xp01=pctile(X), p(1)\
	egen Xp99=pctile(X), p(99)\
	replace smpl=0 if X~=. & (X>Xp99 | X<Xp01);
sum dpreddisbex0gdp dyy diprivgdp digovgdp if smpl==1,d;


/***********************************************************************************************************/
/* Define core regression sample where dy, dg, dpred non-missing. The code up to this point is a modification
/ of the Kraay data with recent GDP data */ 
/***********************************************************************************************************/
gen regsmpl=0;replace regsmpl=1 if smpl==1 & dpreddisbex0gdp~=. & diprivgdp~=. & digovgdp~=.;

/***********************************************************************************************************/
/* Define high-disbursement sample */
/***********************************************************************************************************/
gen igov_ycd = igov/(xr*weogdpcd);
gen disbgov=disbtoty/igov_ycd;
egen xx=mean(disbgov) if regsmpl==1, by(ctry);egen disbgovav=mean(xx), by(ctry);drop xx;
egen xx=mean(disbtotgdp) if regsmpl==1, by(ctry);egen disbtotgdpav=mean(xx), by(ctry);drop xx;
egen xx=sum(regsmpl) if regsmpl==1, by(ctry);egen smplnobs=mean(xx), by(ctry);drop xx;
summarize disbgovav if regsmpl==1, d;

/* Indicator for high disbursement country sample i.e. disbgovav>0.1*/
gen highdisb=1 if disbgovav>0.1;

/**Generate income groups based on WB classifications: 1: LICs 2.LMICs 3. UMICs**/
egen cid = group(income_wdi);

/*** Generate new balanced panel id ***/
drop id;
egen id = group(ctry);
summ id;
sort id year;

log using FKU2024_CrowdingIn.smcl, replace;

/** Table 1 Summary Statistics **/
gen diprivgdp_p = diprivgdp*100;
gen digovgdp_p = digovgdp*100;
gen dpreddisbex0gdp_p = dpreddisbex0gdp*100;
gen dmpreddisbex0gdp_p = dmpreddisbex0gdp*100;

asdoc tabstat diprivgdp_p digovgdp_p dpreddisbex0gdp_p dmpreddisbex0gdp_p, stat(mean median sd min max iqr skewness kurtosis count) col(stat) save(Table_1.doc) replace;

/*** Transform dependent variable for Local Projection specification **/
gen F1diprivgdp = (F1.ipriv/F1.pgdp-L.ipriv/L.pgdp)/L.weogdpkl;
gen F2diprivgdp = (F2.ipriv/F2.pgdp-L.ipriv/L.pgdp)/L.weogdpkl;
gen F3diprivgdp = (F3.ipriv/F3.pgdp-L.ipriv/L.pgdp)/L.weogdpkl;
gen F4diprivgdp = (F4.ipriv/F4.pgdp-L.ipriv/L.pgdp)/L.weogdpkl;


/** Table 2 (Panel A): OLS Estimates using Discroll-Kraay Standard Errors a lá Boehm (2020)*/
xi: xtscc diprivgdp digovgdp i.ctry i.year  if regsmpl==1,  lag(2);
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_2_Panel_A.xls, replace  bdec(3) keep(digovgdp) excel lab ctitle(Baseline)  addstat(No. of countries, `r(max)') nor2 nocons;
xtcd2, seed(12);
drop N_g;

xi: xtscc diprivgdp digovgdp L1diprivgdp i.ctry i.year  if regsmpl==1, lag(2);
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_2_Panel_A.xls, append  bdec(3) keep(digovgdp) excel lab ctitle(h=0)  addstat(No. of countries, `r(max)') nor2 nocons;
xtcd2, seed(12);
drop N_g;

xi: xtscc F1diprivgdp digovgdp L1diprivgdp i.ctry i.year  if regsmpl==1,  lag(2);
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_2_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=1)  addstat(No. of countries, `r(max)') nor2 nocons;
 xtcd2, seed(12);
drop N_g;

xi: xtscc F2diprivgdp digovgdp L1diprivgdp i.ctry i.year  if regsmpl==1,  lag(2);
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_2_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=2)  addstat( No. of countries, `r(max)') nor2 nocons;
 xtcd2, seed(12);
drop N_g;

xi: xtscc F3diprivgdp digovgdp L1diprivgdp i.ctry i.year  if regsmpl==1,  lag(2);
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_2_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=3)  addstat(No. of countries, `r(max)') nor2 nocons;
 xtcd2, seed(12);
drop N_g;

xi: xtscc F4diprivgdp digovgdp L1diprivgdp i.ctry i.year  if regsmpl==1,  lag(2);
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_2_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=4)  addstat( No. of countries, `r(max)') nor2 nocons;
 xtcd2, seed(12);
drop N_g;

/** Table 2 (Panel B): 2SLS Estimates using Discroll-Kraay Standard Errors a lá Boehm (2020)*/
xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0gdp) i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_2_Panel_B.xls, replace  bdec(3) keep(digovgdp) excel lab ctitle(Baseline)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_2_Panel_B.xls, append  bdec(3) keep(digovgdp) excel lab ctitle(h=0)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F1diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_2_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=1)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F2diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_2_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=2)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F3diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_2_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=3)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F4diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_2_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=4)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

/*** Table 3: Estimates for Low-income Countries ***/
xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & cid==1,  dkraay(1) first ;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_3_LICs.xls, replace bdec(3) keep(digovgdp) excel lab ctitle(h=0)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries,`r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F1diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & cid==1,  dkraay(1) first ;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_3_LICs.xls, append bdec(3) keep(digovgdp) excel lab ctitle(h=1)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F2diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & cid==1,  dkraay(1) first ;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_3_LICs.xls, append bdec(3) keep(digovgdp) excel lab ctitle(h=2)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F3diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & cid==1,  dkraay(1) first ;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_3_LICs.xls, append bdec(3) keep(digovgdp) excel lab ctitle(h=3)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F4diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & cid==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_3_LICs.xls, append bdec(3) keep(digovgdp) excel lab ctitle(h=4)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

/*** Table 3: Estimates for IDA countries ***/
xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & ida==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_3_IDA.xls, replace bdec(3) keep(digovgdp) excel lab ctitle(h=0)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F1diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & ida==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_3_IDA.xls, append bdec(3) keep(digovgdp) excel lab ctitle(h=1)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F2diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1  & ida==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_3_IDA.xls, append bdec(3) keep(digovgdp) excel lab ctitle(h=2)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F3diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1  & ida==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_3_IDA.xls, append bdec(3) keep(digovgdp) excel lab ctitle(h=3)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F4diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1  & ida==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_3_IDA.xls, append bdec(3) keep(digovgdp) excel lab ctitle(h=4)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

/* Table 4: Estimates for Sub-Sahara Africa countries */
xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & region_name=="ssa",  dkraay(1) first ;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_3_SSA.xls, replace bdec(3) keep(digovgdp) excel lab ctitle(h=0)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries,`r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F1diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & region_name=="ssa",  dkraay(1) first ;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_3_SSA.xls, append bdec(3) keep(digovgdp) excel lab ctitle(h=1)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F2diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & region_name=="ssa",  dkraay(1) first ;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_3_SSA.xls, append bdec(3) keep(digovgdp) excel lab ctitle(h=2)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F3diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & region_name=="ssa",  dkraay(1) first ;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_3_SSA.xls, append bdec(3) keep(digovgdp) excel lab ctitle(h=3)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F4diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & region_name=="ssa",  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_3_SSA.xls, append bdec(3) keep(digovgdp) excel lab ctitle(h=4)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;


/* Table 4 */
/* Create country specific decades a lá Kraay (2014) */
gen decade="1980s" if year >=1980; 
replace decade="1990s" if year>=1990 & year<=1999;
replace decade="2000s" if year>=2000 & year<=2009;
replace decade="2010s" if year>=2010 ;

egen ctrydecade=concat(ctry decade); 
egen cceav=mean(cce) if regsmpl==1, by(ctrydecade);
egen cceav50=pctile(cceav) , p(50);

/**** Table 4: Estimates for Low corruption control****/
xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & cceav<cceav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_4_cce_low.xls, replace  bdec(3) keep(digovgdp) excel lab ctitle($h=0$)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F1diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & cceav<cceav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_4_cce_low.xls, append  bdec(3) keep(digovgdp) excel lab ctitle($h=1$)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F2diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & cceav<cceav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_4_cce_low.xls, append  bdec(3) keep(digovgdp) excel lab ctitle($h=2$)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F3diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & cceav<cceav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_4_cce_low.xls, append  bdec(3) keep(digovgdp) excel lab ctitle($h=3$)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F4diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & cceav<cceav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_4_cce_low.xls, append  bdec(3) keep(digovgdp) excel lab ctitle($h=4$)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;


/*****Table 4: Estimates for High corruption control*****/
xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & cceav>=cceav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_4_cce_high.xls, replace  bdec(3) keep(digovgdp) excel lab ctitle($h=0$)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F1diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & cceav>=cceav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_4_cce_high.xls, append  bdec(3) keep(digovgdp) excel lab ctitle($h=1$)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F2diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & cceav>=cceav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_4_cce_high.xls, append  bdec(3) keep(digovgdp) excel lab ctitle($h=2$)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F3diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & cceav>=cceav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_4_cce_high.xls, append  bdec(3) keep(digovgdp) excel lab ctitle($h=3$)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F4diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & cceav>=cceav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_4_cce_high.xls, append  bdec(3) keep(digovgdp) excel lab ctitle($h=4$)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

/**** Table 5: Estimates for Low Financial Development */
 xtile fdav=FD if regsmpl==1,  nq(2);
xtile fdav_l=FD if regsmpl==1 & cid==1,  nq(2);
xtile fdav_ida=FD if regsmpl==1 & ida==1,  nq(2);
xtile fdav_h=FD if regsmpl==1 & highdisb==1,  nq(2);

xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fdav ==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_5_FD_low.xls, replace  bdec(3) keep(digovgdp) excel lab ctitle(h=0)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F1diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fdav==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_5_FD_low.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=1)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F2diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fdav ==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_5_FD_low.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=2)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F3diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fdav==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_5_FD_low.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=3)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;


xi: ivreg2 F4diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fdav==1 ,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_5_FD_low.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=4)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

/**** Table 5: Estimates for High Financial Development */
xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fdav ==2 ,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_5_FD_high.xls, replace bdec(3) keep(digovgdp)  lab ctitle(h=0)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F1diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fdav ==2,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_5_FD_high.xls, append  bdec(3) keep(digovgdp) excel lab ctitle(h=1)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F2diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fdav==2,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_5_FD_high.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=2)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F3diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fdav==2,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_5_FD_high.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=3)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F4diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fdav==2,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_5_FD_high.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=4)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;


/**** Table 6 (Panel A): 2SLS estimates using  Multilateral Disbursement as IV***/
xi: ivreg2 diprivgdp (digovgdp = dmpreddisbex0gdp) i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_6_Panel_A.xls, replace  bdec(3) keep(digovgdp) excel lab ctitle(Baseline)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 diprivgdp (digovgdp = dmpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_6_Panel_A.xls, append  bdec(3) keep(digovgdp) excel lab ctitle(h=0)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F1diprivgdp (digovgdp = dmpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_6_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=1)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F2diprivgdp (digovgdp = dmpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_6_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=2)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F3diprivgdp (digovgdp = dmpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_6_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=3)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F4diprivgdp (digovgdp = dmpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_6_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=4)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

/**** Table 6 (Panel B): 2SLS estimates using  Bilateral Disbursement as IV***/
xi: ivreg2 diprivgdp (digovgdp = dbpreddisbex0gdp) i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_6_Panel_B.xls, replace  bdec(3) keep(digovgdp) excel lab ctitle(Baseline)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 diprivgdp (digovgdp = dbpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_6_Panel_B.xls, append  bdec(3) keep(digovgdp) excel lab ctitle(h=0)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F1diprivgdp (digovgdp = dbpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_6_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=1)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F2diprivgdp (digovgdp = dbpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_6_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=2)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F3diprivgdp (digovgdp = dbpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_6_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=3)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F4diprivgdp (digovgdp = dbpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_6_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=4)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

/*** Table 7: Controlling for strong cross sectional dependence a lá Chudik et al. (2013, 2017)***/
egen dpm   = mean(diprivgdp), by(year);
egen dgm   = mean(digovgdp), by(year);
egen F1dpm = mean(F1diprivgdp), by(year);
egen F2dpm = mean(F2diprivgdp), by(year);
egen F3dpm = mean(F3diprivgdp), by(year);
egen F4dpm = mean(F4diprivgdp), by(year);

xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0gdp) i.ctry L(0/3).dgm L(0/3).dpm  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_7.xls, replace  bdec(3) keep(digovgdp) excel lab ctitle(Baseline)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry L(0/3).dgm L(0/3).dpm  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_7.xls, append  bdec(3) keep(digovgdp) excel lab ctitle(h=0)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F1diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry L(0/3).dgm L(0/3).F1dpm  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_7.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=1)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F2diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry L(0/3).dgm L(0/3).F2dpm  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_7.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=2)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F3diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry L(0/3).dgm L(0/3).F3dpm  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_7.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=3)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F4diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry L(0/3).dgm L(0/3).F4dpm  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_7.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=4)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

/*** Figure 3 and 4 *******************/
/*Exogeneity and Exclusion restriction*/
/**************************************/
/* Figure 3 Panel A */
xi: kinkyreg diprivgdp (digovgdp = dpreddisbex0gdp) i.ctry i.year  if regsmpl==1, range(-0.7 0.1) inference(digovgdp) coefplot(iv digovgdp, lpattern(solid) lcolor(black)) coefplot(kls digovgdp, lpattern(dash) lcolor(black)) ciplot(iv digovgdp, fcolor(gray) fintensity(40)  alwidth(none) ) ciplot(kls digovgdp, fcolor(ltblue)  fintensity(40)  alwidth(none) ) twoway(digovgdp, ytitle("Crowding-in effect {&beta} ", size(medium)) yscale(titlegap(*30)) xtitle("Postulated endogeneity of public investment ({&rho})", size(medium)) xlabel(-0.7(0.1)0.1)  legend(position(6) region(lwidth(none)) size(small) cols(4)));
graph export "${outputs}\fig3_Panel_A.png", replace;


/* Figure 4 Panel A */
estat excl, twoway(, ytitle("P-value ", size(medium)) xtitle("Postulated endogeneity of public investment ({&rho})", size(medium)) ylabel(, labsize(medium)) xlabel(-0.7(0.1)0.1, labsize(medium)) legend(position(6) region(lwidth(none)) size(medium) cols(4)));
graph export "${outputs}\fig4_Panel_A.png", replace;

/* Figure 3 Panel B */
xi: kinkyreg diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1, range(-0.7 0.1) inference(digovgdp) coefplot(iv digovgdp, lpattern(solid) lcolor(black)) coefplot(kls digovgdp, lpattern(dash) lcolor(black)) ciplot(iv digovgdp, fcolor(gray) fintensity(40)  alwidth(none) ) ciplot(kls digovgdp, fcolor(ltblue)  fintensity(40)  alwidth(none)) twoway(digovgdp, ytitle("Crowding-in effect {&beta} ", size(medium)) yscale(titlegap(*30)) xtitle("Postulated endogeneity of public investment ({&rho})", size(medium)) xlabel(-0.7(0.1)0.1)  legend(position(6) region(lwidth(none)) size(medium) cols(4)));
graph export "${outputs}\fig3_Panel_B.png", replace;

/* Figure 4 Panel B */
estat excl, twoway(, ytitle("P-value ", size(medium)) xtitle("Postulated endogeneity of public investment ({&rho})", size(medium)) ylabel(, labsize(medium)) xlabel(-0.7(0.1)0.1, labsize(medium)) legend(position(6) region(lwidth(none)) size(small) cols(4)));
graph export "${outputs}\fig4_Panel_B.png", replace;

/* Figure 3 Panel C */
xi: kinkyreg F1diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1, range(-0.7 0.1) inference(digovgdp) coefplot(iv digovgdp, lpattern(solid) lcolor(black)) coefplot(kls digovgdp, lpattern(dash) lcolor(black)) ciplot(iv digovgdp, fcolor(gray) fintensity(40)  alwidth(none) ) ciplot(kls digovgdp, fcolor(ltblue)  fintensity(40)  alwidth(none)) twoway(digovgdp, ytitle("Crowding-in effect {&beta} ", size(medium)) yscale(titlegap(*30)) xtitle("Postulated endogeneity of public investment ({&rho})", size(medium)) ylabel(, labsize(medium)) xlabel(-0.7(0.1)0.1, labsize(medium)) legend(position(6) region(lwidth(none)) size(small) cols(4)));
graph export "${outputs}\fig3_Panel_C.png", replace;

/* Figure 4 Panel C */
estat excl, twoway(, ytitle("P-value ", size(medium)) xtitle("Postulated endogeneity of public investment ({&rho})", size(medium)) ylabel(, labsize(medium)) xlabel(-0.7(0.1)0.1, labsize(medium)) legend(position(6) region(lwidth(none)) size(medium) cols(4)));
graph export "${outputs}\fig4_Panel_C.png", replace;


/* Figure 3 Panel D */
xi: kinkyreg F2diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1, range(-0.7 0.1) inference(digovgdp) coefplot(iv digovgdp, lpattern(solid) lcolor(black)) coefplot(kls digovgdp, lpattern(dash) lcolor(black)) ciplot(iv digovgdp, fcolor(gray) fintensity(40)  alwidth(none) ) ciplot(kls digovgdp, fcolor(ltblue)  fintensity(40)  alwidth(none)) twoway(digovgdp, ytitle("Crowding-in effect {&beta} ", size(medium)) yscale(titlegap(*30)) xtitle("Postulated endogeneity of public investment ({&rho})", size(medium)) ylabel(, labsize(medium)) xlabel(-0.7(0.1)0.1, labsize(medium)) legend(position(6) region(lwidth(none)) size(small) cols(4)));
graph export "${outputs}\fig3_Panel_D.png", replace;

/* Figure 4 Panel D */
estat excl, twoway(, ytitle("P-value", size(medium)) xtitle("Postulated endogeneity of public investment ({&rho})", size(medium)) ylabel(, labsize(medium)) xlabel(-0.7(0.1)0.1, labsize(medium)) legend(position(6) region(lwidth(none)) size(medium) cols(4)));
graph export "${outputs}\fig4_Panel_D.png", replace;

/* Figure 3 Panel E*/
xi: kinkyreg F3diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1, range(-0.7 0.1) inference(digovgdp) coefplot(iv digovgdp, lpattern(solid) lcolor(black)) coefplot(kls digovgdp, lpattern(dash) lcolor(black)) ciplot(iv digovgdp, fcolor(gray) fintensity(40)  alwidth(none) ) ciplot(kls digovgdp, fcolor(ltblue)  fintensity(40)  alwidth(none)) twoway(digovgdp, ytitle("Crowding-in effect {&beta} ", size(medium)) yscale(titlegap(*30)) xtitle("Postulated endogeneity of public investment ({&rho})", size(medium)) ylabel(, labsize(medium)) xlabel(-0.7(0.1)0.1, labsize(medium)) legend(position(6) region(lwidth(none)) size(small) cols(4)));
graph export "${outputs}\fig3_Panel_E.png", replace;

/* Figure 4 Panel E */
estat excl, twoway(, ytitle("P-value", size(medium)) xtitle("Postulated endogeneity of public investment ({&rho})", size(medium)) ylabel(, labsize(medium)) xlabel(-0.7(0.1)0.1, labsize(medium)) legend(position(6) region(lwidth(none)) size(medium) cols(4)));
graph export "${outputs}\fig4_Panel_E.png", replace;

/* Figure 3 Panel F */
xi: kinkyreg F4diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1, range(-0.7 0.1) inference(digovgdp) coefplot(iv digovgdp, lpattern(solid) lcolor(black)) coefplot(kls digovgdp, lpattern(dash) lcolor(black)) ciplot(iv digovgdp, fcolor(gray) fintensity(40)  alwidth(none) ) ciplot(kls digovgdp, fcolor(ltblue)  fintensity(40)  alwidth(none)) twoway(digovgdp, ytitle("Crowding-in effect {&beta} ", size(medium)) yscale(titlegap(*30)) xtitle("Postulated endogeneity of public investment ({&rho})", size(medium)) ylabel(, labsize(medium)) xlabel(-0.7(0.1)0.1, labsize(medium)) legend(position(6) region(lwidth(none)) size(small) cols(4)));
graph export "${outputs}\fig3_Panel_F.png", replace;

/* Figure 4 Panel F */
estat excl, twoway(, ytitle("P-value", size(medium)) xtitle("Postulated endogeneity of public investment ({&rho})", size(medium)) ylabel(, labsize(medium)) xlabel(-0.7(0.1)0.1, labsize(medium)) legend(position(6) region(lwidth(none)) size(medium) cols(4)));
graph export "${outputs}\fig4_Panel_F.png", replace;

/***************************************************************/
/*Annex Estimation Results                                     */
/***************************************************************/

/*** ANNEX C Table C1****/
/* Instrumental variable with alternative censoring rules */ 
/*---------------------------------------------------------------*/
/**Table C1: Total disbursements to loan > 2 **/
xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0_rdsum2gdp) i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using Table_C1_Panel_A.xls, replace  bdec(3) keep(digovgdp) excel lab ctitle(Baseline)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0_rdsum2gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using Table_C1_Panel_A.xls, append  bdec(3) keep(digovgdp) excel lab ctitle(h=0)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F1diprivgdp (digovgdp = dpreddisbex0_rdsum2gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using Table_C1_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=1)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F2diprivgdp (digovgdp = dpreddisbex0_rdsum2gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using Table_C1_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=2)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F3diprivgdp (digovgdp = dpreddisbex0_rdsum2gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using Table_C1_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=3)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F4diprivgdp (digovgdp = dpreddisbex0_rdsum2gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using Table_C1_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=4)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

/**Table C1: Creditors with total loans > 200 **/
xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0_nloans207gdp) i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using Table_C1_Panel_B.xls, replace  bdec(3) keep(digovgdp) excel lab ctitle(Baseline)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0_nloans207gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using Table_C1_Panel_B.xls, append  bdec(3) keep(digovgdp) excel lab ctitle(h=0)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F1diprivgdp (digovgdp = dpreddisbex0_nloans207gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using Table_C1_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=1)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F2diprivgdp (digovgdp = dpreddisbex0_nloans207gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using Table_C1_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=2)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F3diprivgdp (digovgdp = dpreddisbex0_nloans207gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using Table_C1_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=3)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F4diprivgdp (digovgdp = dpreddisbex0_nloans207gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using Table_C1_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=4)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

/**Table C1: Disb. < 4% of GDP and share disb. < 25% **/
xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0_occdy4gdp) i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using Table_C1_Panel_C.xls, replace  bdec(3) keep(digovgdp) excel lab ctitle(Baseline)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0_occdy4gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using Table_C1_Panel_C.xls, append  bdec(3) keep(digovgdp) excel lab ctitle(h=0)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F1diprivgdp (digovgdp = dpreddisbex0_occdy4gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using Table_C1_Panel_C.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=1)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F2diprivgdp (digovgdp = dpreddisbex0_occdy4gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using Table_C1_Panel_C.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=2)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F3diprivgdp (digovgdp = dpreddisbex0_occdy4gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using Table_C1_Panel_C.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=3)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

xi: ivreg2 F4diprivgdp (digovgdp = dpreddisbex0_occdy4gdp) L1diprivgdp i.ctry i.year  if regsmpl==1,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using Table_C1_Panel_C.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=4)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; xtcd2, seed(12);
drop N_g;

/*** ANNEX C Table C2****/
/* Clustering at the country level */ 
/**Table C2: OLS Results**/
xi: reg diprivgdp digovgdp i.ctry i.year  if regsmpl==1,  robust cluster(id);
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_C2_Panel_A.xls, replace  bdec(3) keep(digovgdp) excel lab ctitle(Baseline)  addstat(No. of countries, `r(max)') nor2 nocons;
drop N_g;

xi: reg diprivgdp digovgdp L1diprivgdp i.ctry i.year  if regsmpl==1, robust cluster(id);
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_C2_Panel_A.xls, append  bdec(3) keep(digovgdp) excel lab ctitle(h=0)  addstat(No. of countries, `r(max)') nor2 nocons;
drop N_g;

xi: reg F1diprivgdp digovgdp L1diprivgdp i.ctry i.year  if regsmpl==1,  robust cluster(id);
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_C2_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=1)  addstat(No. of countries, `r(max)') nor2 nocons;
drop N_g;

xi: reg F2diprivgdp digovgdp L1diprivgdp i.ctry i.year  if regsmpl==1,  robust cluster(id);
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_C2_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=2)  addstat( No. of countries, `r(max)') nor2 nocons;
drop N_g;

xi: reg F3diprivgdp digovgdp L1diprivgdp i.ctry i.year  if regsmpl==1,  robust cluster(id);
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_C2_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=3)  addstat(No. of countries, `r(max)') nor2 nocons;
drop N_g;

xi: reg F4diprivgdp digovgdp L1diprivgdp i.ctry i.year  if regsmpl==1,  robust cluster(id);
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_C2_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=4)  addstat( No. of countries, `r(max)') nor2 nocons;
drop N_g;

/**Table C2: 2SLS results***/
xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0gdp) i.ctry i.year  if regsmpl==1, robust cluster(id) first;
outreg2 using  Table_C2_Panel_B.xls, replace  bdec(3) keep(digovgdp) excel lab ctitle(Baseline)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, e(N_clust)) nor2 nocons;
rivtest, ci; xtcd2, seed(12);

xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1, robust cluster(id) first;
outreg2 using  Table_C2_Panel_B.xls, append  bdec(3) keep(digovgdp) excel lab ctitle(h=0)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, e(N_clust)) nor2 nocons;
rivtest, ci; xtcd2, seed(12);

xi: ivreg2 F1diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1, robust cluster(id) first;
outreg2 using  Table_C2_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=1)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, e(N_clust)) nor2 nocons;
rivtest, ci; xtcd2, seed(12);

xi: ivreg2 F2diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1, robust cluster(id) first;
outreg2 using  Table_C2_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=2)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, e(N_clust)) nor2 nocons;
rivtest, ci; xtcd2, seed(12);

xi: ivreg2 F3diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1, robust cluster(id) first;
outreg2 using  Table_C2_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=3)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, e(N_clust)) nor2 nocons;
rivtest, ci; xtcd2, seed(12);

xi: ivreg2 F4diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1, robust cluster(id) first;
outreg2 using  Table_C2_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=4)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, e(N_clust)) nor2 nocons;
rivtest, ci; xtcd2, seed(12);



/*** ANNEX D Table D1****/
/* Financial Openness a la Chinn-Ito kaopen_2021*/ 
/*---------------------------------------------------------------*/
egen ctrydecade_r=concat(ctry decade); 
egen fpoav=mean(ka_open) if regsmpl==1, by(ctrydecade_r);
egen fpoav50=pctile(fpoav) , p(50);

/* Table D1: Chinn-Ito Low */
xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0gdp)  i.ctry i.year  if regsmpl==1 & fpoav < fpoav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_D1_Panel_A.xls, replace  bdec(3) keep(digovgdp) excel lab ctitle(Baseline)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fpoav < fpoav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_D1_Panel_A.xls, append  bdec(3) keep(digovgdp) excel lab ctitle(h=0)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F1diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fpoav < fpoav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_D1_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=1)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F2diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fpoav < fpoav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_D1_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=2)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F3diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fpoav < fpoav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_D1_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=3)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;


xi: ivreg2 F4diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fpoav < fpoav50 ,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_D1_Panel_A.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=4)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;


/* Table D1: Chinn-Ito High */
xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0gdp)  i.ctry i.year  if regsmpl==1 & fpoav >= fpoav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_D1_Panel_B.xls, replace bdec(3) keep(digovgdp)  lab ctitle(Baseline)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fpoav >= fpoav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_D1_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=0)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F1diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fpoav >= fpoav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_D1_Panel_B.xls, append  bdec(3) keep(digovgdp) excel lab ctitle(h=1)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F2diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fpoav >= fpoav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_D1_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=2)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F3diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fpoav >= fpoav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_D1_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=3)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

xi: ivreg2 F4diprivgdp (digovgdp = dpreddisbex0gdp) L1diprivgdp i.ctry i.year  if regsmpl==1 & fpoav >= fpoav50,  dkraay(1) first;
qui: egen N_g = group(country) if e(sample);
qui: summ N_g;
outreg2 using  Table_D1_Panel_B.xls, append bdec(3) keep(digovgdp)  lab ctitle(h=4)  addstat(F-statistic, e(widstat), KP Underid stat, e(idp), No. of countries, `r(max)') nor2 nocons;
rivtest, ci; 
drop N_g;

log close;





